@@ -323,6 +323,10 @@ int acpi_read (int battery, apm_info *info) {
 		if (state) {
 			if (state[0] == 'd') { /* discharging */
 				info->battery_status = BATTERY_STATUS_CHARGING;
+				/* Expensive ac power check used here
+				 * because AC power might be on even if a
+				 * battery is discharging in some cases. */
+				info->ac_line_status = on_ac_power();
 			}
 			else if (state[0] == 'c' && state[1] == 'h') { /* charging */
 				info->battery_status = BATTERY_STATUS_CHARGING;
@@ -339,6 +343,10 @@ int acpi_read (int battery, apm_info *info) {
 			}
 			else if (state[0] == 'c') { /* not charging, so must be critical */
 				info->battery_status = BATTERY_STATUS_CRITICAL;
+				/* Expensive ac power check used here
+				 * because AC power might be on even if a
+				 * battery is critical in some cases. */
+				info->ac_line_status = on_ac_power();
 			}
 			else if (rate == 0) {
 				/* if rate is null, battery charged, on
@@ -378,7 +386,7 @@ int acpi_read (int battery, apm_info *info) {
 		info->battery_time = 0;
 		info->battery_status = BATTERY_STATUS_ABSENT;
 		acpi_batt_capacity[battery] = 0;
-		if (acpi_batt_count == 1) {
+		if (acpi_batt_count == 0) {
 			/* Where else would the power come from, eh? ;-) */
 			info->ac_line_status = 1;
 		}
